"""
<Program Name>
  sphincs_keys.py

<Author>
  Ruben Gonzalez <mail@ruben-gonzalez.de>

<Started>
  Otober 12, 2022.

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  The goal of this module is to include SPHINCS+ post-quantum signature support.
 """
# 'os' required to generate OS-specific randomness (os.urandom) suitable for
# cryptographic use.
# http://docs.python.org/2/library/os.html#miscellaneous-functions
import os

from securesystemslib import exceptions, formats

SPX_AVAIL = True
NO_SPX_MSG = "spinhcs+ key support requires the pyspx library"

try:
    from pyspx import shake_128s
except ImportError:
    SPX_AVAIL = False

_SHAKE_SEED_LEN = 48


def generate_public_and_private():
    """Generates spx public and private key.

    Returns:
        tuple: Containing the (public, private) keys.
    Raises:
        UnsupportedLibraryError: In case pyspx is not available.
    """
    if not SPX_AVAIL:
        raise exceptions.UnsupportedLibraryError(NO_SPX_MSG)
    seed = os.urandom(_SHAKE_SEED_LEN)
    public, private = shake_128s.generate_keypair(seed)
    return public, private


def create_signature(public_key, private_key, data, scheme):
    """Signs data with the private key.
    Arguments:
          public_key (bytes): The public key. Not used so far.
          private_key (bytes): The private key.
          data (bytes): The data to be signed.
          scheme (str): The name of the scheme as defined in formats.py.
    Returns:
        tuple: Containing the values (signature, scheme).
    Raises:
        UnsupportedLibraryError: In case pyspx is not available.
    """
    if not SPX_AVAIL:
        raise exceptions.UnsupportedLibraryError(NO_SPX_MSG)
    formats.SPHINCSPUBLIC_SCHEMA.check_match(public_key)
    formats.SPHINCSPRIVATE_SCHEMA.check_match(private_key)
    formats.SPHINCS_SIG_SCHEMA.check_match(scheme)

    signature = shake_128s.sign(data, private_key)

    return signature, scheme


def verify_signature(public_key, scheme, signature, data):
    """Verify a signature using the public key.
    Arguments:
          public_key (bytes): The public key used for verification.
          scheme (str): The name of the scheme as defined in formats.py.
          signature (bytes): The sphincs+ signature as generated with create_signature.
          data (bytes): The data that was signed.
    Returns:
        bool: True if the signature was valid, False otherwise.
    Raises:
        UnsupportedLibraryError: In case pyspx is not available.
    """
    if not SPX_AVAIL:
        raise exceptions.UnsupportedLibraryError(NO_SPX_MSG)
    formats.SPHINCSPUBLIC_SCHEMA.check_match(public_key)

    # Is 'scheme' properly formatted?
    formats.SPHINCS_SIG_SCHEMA.check_match(scheme)

    # Is 'signature' properly formatted?
    formats.SPHINCSSIGNATURE_SCHEMA.check_match(signature)

    return shake_128s.verify(data, signature, public_key)



#!/usr/bin/env python
"""
<Program Name>
  check_public_interfaces_gpg.py

<Author>
  Lukas Puehringer <lukas.puehringer@nyu.edu>

<Started>
  Feb 26, 2020.

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Check that the public facing 'gpg.functions' module remains importable if
  gnupg is not installed, and that each function presents meaningful
  user-feedback.
  Further check that gpg signature verification works even without gpg.

  NOTE: the filename is purposefully check_ rather than test_ so that test
  discovery doesn't find this test module and the test cases within are only
  run when explicitly invoked.

"""

import unittest

from securesystemslib.exceptions import UnsupportedLibraryError
from securesystemslib.gpg.constants import NO_GPG_MSG, have_gpg
from securesystemslib.gpg.functions import (
    create_signature,
    export_pubkey,
    export_pubkeys,
    verify_signature,
)
from securesystemslib.gpg.util import get_version


class TestPublicInterfacesGPG(
    unittest.TestCase
):  # pylint: disable=missing-class-docstring
    @classmethod
    def setUpClass(cls):
        assert (
            not have_gpg()
        ), "please remove GnuPG from your environment to run this test case"

    def test_gpg_functions(self):
        """Signing, key export and util functions must raise on missing gpg."""
        with self.assertRaises(UnsupportedLibraryError) as ctx:
            create_signature("bar")
        self.assertEqual(NO_GPG_MSG, str(ctx.exception))

        with self.assertRaises(UnsupportedLibraryError) as ctx:
            export_pubkey("f00")
        self.assertEqual(NO_GPG_MSG, str(ctx.exception))

        with self.assertRaises(UnsupportedLibraryError) as ctx:
            export_pubkeys(["f00"])
        self.assertEqual(NO_GPG_MSG, str(ctx.exception))

        with self.assertRaises(UnsupportedLibraryError) as ctx:
            get_version()
        self.assertEqual(NO_GPG_MSG, str(ctx.exception))

    def test_gpg_verify(self):
        """Signature verification does not require gpg to be installed on the host.
        To prove it, we run basic verification tests for rsa, dsa and eddsa with
        pre-generated/exported signatures and keys. More thorough testing is
        available in test_gpg.py

        """
        data = b"deadbeef"
        key_signature_pairs = [
            # RSA
            (
                {
                    "method": "pgp+rsa-pkcsv1.5",
                    "type": "rsa",
                    "hashes": ["pgp+SHA2"],
                    "creation_time": 1519661780,
                    "keyid": "c5a0abe6ec19d0d65f85e2c39be9df5131d924e9",
                    "keyval": {
                        "private": "",
                        "public": {
                            "e": "010001",
                            "n": "c152fc1f1535a6d3c1e8c0dece7f0a1d09324466e10e4ea51d5d7223ab125c1743393eebca73ccb1022d44c379fae30ef63b263d0a793882a7332ef06f28a4b9ae777f5d2d8d289167e86c162df1b9a9e127acb26803688556ecb08492d071f06caf88cea95571354349d8ef131eff03b0d259fae30ebf8dac9ab5acd6f26f4770fe2f30fcd0a3c54f03463a3094aa6524e39027a625108f04e12475da248fb3b536df61b0f6e2954739b8828c61171f66f8e176823e1c887e65fa0aec081013b2a50ed60515f7e3b3291ca443e1222b9b625005dba045a7208188fb88d436d473f6340348953e891354c7a5734bf64e6274e196db3074a7ce3607960baacb1b",
                        },
                    },
                },
                {
                    "keyid": "c5a0abe6ec19d0d65f85e2c39be9df5131d924e9",
                    "other_headers": "04000108001d162104c5a0abe6ec19d0d65f85e2c39be9df5131d924e905025e56444b",
                    "signature": "bc4490901bd6edfe0ec49e0358c0a7ef37fc229824ca75dd4f163205745c78baaa2ca5cda79be259a5ac8323b4c1a1ee18fab0a8cc90eeafeb3eb1221d4bafb55510f34cf99e7ac121874f3c01152d6d8953c661c3e5147a387fffaee672318ed39c49fa02c80fa806956695f2fdfe0429a61639e7fb544f1531100eb02b7a140ffa284746fa1620e8461e4af5f93594f8aed6d34a33d51b265bae90ea8bedccb7497594003eb46516bddb1778a4fadd02cbb227e1931eeb5ef445fb9745f85cfbebfa169c3ae7d15e2ca75b15dd020877c9a968ff853993a06420d3c3ff158800014f21e558103cd4e7e84cf5e320ebf7c525e0eab9ab22ad4af02c7ad48b5e",
                },
            ),
            # DSA
            (
                {
                    "method": "pgp+dsa-fips-180-2",
                    "type": "dsa",
                    "hashes": ["pgp+SHA2"],
                    "creation_time": 1510870182,
                    "keyid": "c242a830daaf1c2bef604a9ef033a3a3e267b3b1",
                    "keyval": {
                        "private": "",
                        "public": {
                            "y": "2dd50b2292441444581f9a0b7d8d7f88b573fc451f5e7207c324694232c22e171b508f6842ae9babc56fe4e586a22086188b4827b7aba8c7bff4a4ac9aa80c835420b1afba4ab4f1b1c0ef894437903a9f4c56ebef037804a99925c9a153b8a16c1562f297755aeaa20fa02ab32aa5366e052b6baa9a934356d4f5fc218785018dd12b2c8e6d605d2afb36cb06a9cced9ea1f5f82798d635de264ef0eb59590c4a4b2fdf2369a36f95614804c7aa5966ba9597404ba2d2c6881959112de52de4b6d4f1e2c8a59ddaadb08a59ac8334118f15aa01593e851024905ea6d884c3a545af6fdd03c8d2b54da1d35e710ef75a2b4775bb78c50b28d1e2fb48416dc941",
                            "p": "fca3276cd78c20e3c73ae2398674046039f5d90f41e3ede9bc99f94000d145693522671fba481d22e0a9b31e695d198da5e62f4ffb4db5dc64076d0f2d7d03ce953fc7846a6d4e17a10bf1dcd17167f7aff761b59fa2180e7fcd2ca527c03c50c78665b5539bf2b45648b6d23f31f37999e6a7b4e0876ddad7ec783b8eec7e1fb14733e74b6b0b105cbdc5a7de8e094657f2146ce43a3177581cb022a4e2ce6678a3364a56e02090559a6dfd81d91ca3b7c6afd4fcfc66fd88339d217062462f51c5c91d6eccfafb32065be68e6b91ec837c59a51baebeca1c70fd3891c9bbb67f7d920f9153fc4d2ca03f88a27b70df1684709f99ad18707189b015441b2bfb",
                            "g": "7f7252ae1824baf2be5fc8f431a1978683a38d4a22cc2bcdc01ccd1f5eee47a964aa57639a618cfb1b10707b4d09ff11a448e83ba70123573f2d49a599f5313a74463e5bb3ca3d6172a00f02b01065ce312501e1797f7b57e606947c44bd839fde8d43269f1fb74af6cedf4db7fabf0b2357ed09d56381ac769ef5a8af1b4450e0c88b64ee1cab9fadeb31b7be6207b7e17008a33a7613831f70a123d59279dcbc2238f46eeaa8097795b7805f1b837ef3b8e807164e186fae9fa3ff510213096bf54040eac545a6a5b47c910e6cf7e306e1f46723f14b02cd9e0b0ff2a56c3b2604869431ab3263d61bf5068bee36c880c7bf2c746dcae5d0d7b2fff244ef43",
                            "q": "84779eeae0238d7a9a030a639bf01a0f9ef517a5d950599c19a4e54fbbf23219",
                        },
                    },
                },
                {
                    "keyid": "c242a830daaf1c2bef604a9ef033a3a3e267b3b1",
                    "other_headers": "04001108001d162104c242a830daaf1c2bef604a9ef033a3a3e267b3b105025e5644d1",
                    "signature": "3044022009e95f952f64f559852fb6b321173f3cb142a5dbe0c84d709d55026ab945582802203144ee0f4c2cb70fa00ca6942c847208b96811271445ed85c75ebebdb609b174",
                },
            ),
            # EDDSA
            (
                {
                    "method": "pgp+eddsa-ed25519",
                    "type": "eddsa",
                    "hashes": ["pgp+SHA2"],
                    "creation_time": 1572269200,
                    "keyid": "4e630f84838bf6f7447b830b22692f5fea9e2dd2",
                    "keyval": {
                        "private": "",
                        "public": {
                            "q": "716e57b8c5d4397a4194f80bd43af2e07691db7ee58d2473ceb56cef1eda7569"
                        },
                    },
                },
                {
                    "keyid": "4e630f84838bf6f7447b830b22692f5fea9e2dd2",
                    "other_headers": "04001608001d1621044e630f84838bf6f7447b830b22692f5fea9e2dd205025e564505",
                    "signature": "70ba3fe785bccac105b837b6b27cc8d5ddd0159c3f640bbac026b744e0b10839bf4ea53e786074d32f9617389a4fe3356ec1c4a19045c5c02821563786e1d10d",
                },
            ),
        ]

        for key, sig in key_signature_pairs:
            self.assertTrue(verify_signature(sig, key, data))


if __name__ == "__main__":
    unittest.main(verbosity=1, buffer=True)



#!/usr/bin/env python

"""
<Program Name>
  check_public_interfaces.py

<Author>
  Joshua Lock <jlock@vmware.com>

<Started>
  January 6, 2020.

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Public facing modules (e.g. interface.py and keys.py) must be
  importable, even if the optional dependencies are not installed.

  Each public facing function should always be callable and present
  meaningful user-feedback if an optional dependency that is required for
  that function is not installed.

  This test purposefully only checks the public functions with a native
  dependency, to avoid duplicated tests.

  NOTE: the filename is purposefully check_ rather than test_ so that test
  discovery doesn't find this unittest and the tests within are only run
  when explicitly invoked.
"""

import os
import shutil
import tempfile
import unittest
from unittest import mock

import securesystemslib.exceptions  # pylint: disable=wrong-import-position
import securesystemslib.gpg.constants  # pylint: disable=wrong-import-position
import securesystemslib.gpg.functions  # pylint: disable=wrong-import-position
import securesystemslib.gpg.util  # pylint: disable=wrong-import-position
import securesystemslib.interface  # pylint: disable=wrong-import-position
import securesystemslib.keys  # pylint: disable=wrong-import-position


class TestPublicInterfaces(
    unittest.TestCase
):  # pylint: disable=missing-class-docstring
    @classmethod
    def setUpClass(cls):
        cls.temp_dir = tempfile.mkdtemp(dir=os.getcwd())

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.temp_dir)

    def test_interface(self):

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.interface._generate_and_write_rsa_keypair(  # pylint: disable=protected-access
                password="pw"
            )

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.interface.generate_and_write_rsa_keypair("pw")

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.interface.generate_and_write_rsa_keypair("pw")

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            # Mock entry on prompt which is presented before lower-level functions
            # raise UnsupportedLibraryError
            with mock.patch(
                "securesystemslib.interface.get_password", return_value=""
            ):
                securesystemslib.interface.generate_and_write_rsa_keypair_with_prompt()

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.interface.generate_and_write_unencrypted_rsa_keypair()

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            path = os.path.join(self.temp_dir, "rsa_key")
            with open(path, "a"):  # pylint: disable=unspecified-encoding
                securesystemslib.interface.import_rsa_privatekey_from_file(path)

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.interface._generate_and_write_ed25519_keypair(  # pylint: disable=protected-access
                password="pw"
            )

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.interface.generate_and_write_ed25519_keypair("pw")

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            # Mock entry on prompt which is presented before lower-level functions
            # raise UnsupportedLibraryError
            with mock.patch(
                "securesystemslib.interface.get_password", return_value=""
            ):
                securesystemslib.interface.generate_and_write_ed25519_keypair_with_prompt()

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.interface.generate_and_write_unencrypted_ed25519_keypair()

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            path = os.path.join(self.temp_dir, "ed25519_priv.json")
            with open(path, "a") as f:  # pylint: disable=unspecified-encoding
                f.write("{}")
                securesystemslib.interface.import_ed25519_privatekey_from_file(
                    path, "pw"
                )

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.interface._generate_and_write_ecdsa_keypair(  # pylint: disable=protected-access
                password="pw"
            )

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.interface.generate_and_write_ecdsa_keypair("pw")

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            # Mock entry on prompt which is presented before lower-level functions
            # raise UnsupportedLibraryError
            with mock.patch(
                "securesystemslib.interface.get_password", return_value=""
            ):
                securesystemslib.interface.generate_and_write_ecdsa_keypair_with_prompt()

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.interface.generate_and_write_unencrypted_ecdsa_keypair()

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            path = os.path.join(self.temp_dir, "ecddsa.priv")
            with open(path, "a") as f:  # pylint: disable=unspecified-encoding
                f.write("{}")
                securesystemslib.interface.import_ecdsa_privatekey_from_file(
                    path, password="pw"
                )

    def test_keys(self):
        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.generate_rsa_key()

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.generate_ecdsa_key()

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.generate_ed25519_key()

        data = "foo"
        keydict = {
            "keytype": "ed25519",
            "scheme": "ed25519",
            "keyid": "f00",
            "keyval": {"private": "f001", "public": "b00f"},
        }
        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.create_signature(keydict, data)

        keydict["keytype"] = "sphincs"
        keydict["scheme"] = "sphincs-shake-128s"
        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.create_signature(keydict, data)

        keydict["keytype"] = "ecdsa"
        keydict["scheme"] = "ecdsa-sha2-nistp256"
        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.create_signature(keydict, data)

        keydict["keytype"] = "rsa"
        keydict["scheme"] = "rsassa-pss-sha256"
        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.create_signature(keydict, data)

        keydict["keytype"] = "ecdsa"
        keydict["scheme"] = "ecdsa-sha2-nistp256"
        sig = {
            "keyid": "f00",
            "sig": "cfbce8e23eef478975a4339036de2335002d57c7b1632dd01e526a3bc52a5b261508ad50b9e25f1b819d61017e7347e912db1af019bf47ee298cc58bbdef9703",
        }
        # NOTE: we don't test ed25519 keys as they can be verified in pure python
        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.verify_signature(keydict, sig, data)

        keydict["keytype"] = "sphincs"
        keydict["scheme"] = "sphincs-shake-128s"
        sig = {
            "keyid": "f00",
            "sig": "A" * 7_856,
        }
        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.verify_signature(keydict, sig, data)

        keydict["keytype"] = "rsa"
        keydict["scheme"] = "rsassa-pss-sha256"
        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.verify_signature(keydict, sig, data)

        priv = "-----BEGIN RSA PRIVATE KEY-----\nMIIG5AIBAAKCAYEA2WC/pM+6/NbOE/b+N9L+5BOa5sLHCF88okpiCJAZhtIEMw8O\n/EX4CjSy5Qilrmj7ZXmwRyPf7ksd6dbgxAJYk555lE2dywdvzsd31B+nKuAky8/K\nNjpfH4bn2sBKxbA9FFrBenpBkBrq0qDyK85VGJO7ieUdjQepiBQbqctU/PxmPJcE\neO0f1X4IjA+MQv6j/Wt+dnCQSFpCHgOEA0CBWByfRR+DIX74y8RYyKHgj+LpNv1A\nUD1K2vbNc/LrZWEIojCz+2QcXtz/g0kXX5DmRP3feGMC/S/r9bIjEdP55XP70LQU\ndaly64Y/nOlwWHhDNRjtu0lfdqxrK30/O8S8NC6A+nXrav1DzOufffd6wuRKiEqc\nEXZGitSyt/Bg5z70jIHgP6sZ69F0uORr3CaX/YAcQdjPzvSkJEvSj1/sSa+iKOPe\nixQx3VoEpdI3wWu7TQBmTOA3gi2XEZFYdThMGUA5Yv/qNHQVHBkEvOdtTRbWFX0m\npBHLTwBoMO+VJI6hAgMBAAECggGATAC5wOQomrJ4Bx76r4YEPLZmGHzNni2+Q3gC\nYsAPTMYtVbTUJnxIRzk5uz6UvzBRhZ9QdO8kImr9IH9SwvWXBrYICERDAXOuMfwn\n93DBwAnyk5gpOWCbVaiTdDZ7bjc6g91ffHU2ay4eIFrJkWto8Vjl30bOWDrvmXZ+\nXZWMN5AAJvseQzGVSc3xKxdckSf7KmXlJ4Af0kxMhbXw+DobfzUysrZb4OBGGOij\nqjJ/E4/gvqs5S1TC0WAtYXbzutR7zVGuZUFVK7Lk1fq8XcJP5wXCrIjxGnP6V97y\nWn1h64eD+7Gt4wQ+IGr0zKxhSYWI4ou+6QIV3kGlFv9ZRI22yym9MalG1Z1g2GP4\nrgcBZ6j87siSG2L5WoA62pxPPm+vfgEW3GYty1sYqVVQEQhy7GGHWT1kYcc0H7Sr\nALspSr3VbDJtylMQ+wl2IHs8qQ2GAW/utHwPyPzgY2wswi/6L8oYKBrEKK66gSlF\nPHek3uSbho2cPVW7RpG3NA5AHJBhAoHBAO48GEnmacBvMwHfhHex6XUX+VW0QxMl\n/8uNbAp4MEgdyqLw1TLUUAvEbV6qOwL3IWxAvJjXl/9zPtiBUiniZfUI7Rm0LMlv\n1jUlXfzuLwZtL8dHUDFBaZNWlY+eG5dniWkhzMnKqYYGbs9DDO741AKWUtM9UtBA\nm6g0AP6maa3RRAFQ+JtoVFuMYg6R4oE621pKI5ZJ1Zmz/L6H1xoj1QH0JPND1Mxa\nqYEj5SAKE+tj4dbsHjKeaPjk30qnlulQPQKBwQDpln8mJ3z7EXGCYMQrbtg94YuR\n/AVM5pZL9V1YNB8jiydg3j5tMjXWSxd+Hc3Kg1Ey0SjWGtPGD1RQQM+ZQgubRFHP\n7RwQwhxwxji5Azl5LoupsNueMGLQ0bBxSQWTx8zxc4z5oVBcZgD4Pm+5wi17L/77\nqM9Md2nw4ONbsxMiNol65dc/XUPuxaUpPAe2XlV4EGsyWDee6OhH288WhOAzpixS\nB1Ywc6f7LNLc065w2rjzogzyONAFkTP4kKe/2jUCgcEAxznuPe64RTs49roLN2XL\nDCcOVgO3jA3dCkasMV0tU0HGsdihEi7G+fA8XkwRqXstsi+5CEBTVkb0KW6MXYZ9\nKRtb3ID2a0ZhZnRnUxuEq+UnbYlPoMFJHvPrgvz/qe/l08t2TNJ0TiaXCDDUYgwo\nkDlR7mF8HbfJ9DH5GvvjqH42Vrt2C9CFq0GMxw5s0xF7WthhRk9cl3sTQ+qpkayh\nd07Kj70L+hFfayWveMm0usb+mBNBdadPtcUAjpfz9g0pAoHBALWdULDOpQrkThfr\nurp2TWUXlxfjFg/rfNIELRZmOAu/ptdXFLx7/IXoDpT9AUNChIB5RUHqy9tDke9v\n5LkpM7L+FIoQtfCFq+03AWVAD5Cb0vUV0DuXLU1kq8X424BCKaNVjzeL59pfaMOa\nb+3C/u+3qo3qe3rdoZ4qjDuA6RCBzLSkPY5DqozcWQTNasWtZNCcG2yiUGSae/da\n/RFqMJOX0P/aOnYjhmjxOeV+JDQUqxaqWVx/NaYOdpT9i5/MPQKBwGaMbFVt0+CR\nRT5Ts/ZS1qCmyoIepFMOI0SyU8h5+qk4dGutXCm1zjyyxwdJAjG1PYny5imsc795\nR7g7PLSUA+pkXWU8aoiCuCkY6IYz8JFLAw74mxZdLaFQUfBBtSqMz4B9YvUOysr1\nj7Og3AYXob4Me1+ueq59YLM9fEd4Tbw+aBg5T27jwZEmmNripamNFFb6RuPq6u6H\nMZW81M7ahgizqGQsRcOskA/uBC1w3N7o/lUYa3I+OY6EqA4KigIuGw==\n-----END RSA PRIVATE KEY-----\n"

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.import_rsakey_from_private_pem("")

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.encrypt_key(keydict, "foo")

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.decrypt_key("enc", "pw")

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.create_rsa_encrypted_pem(priv, "pw")

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.import_ed25519key_from_private_json(
                "".encode("utf-8"), ""
            )

        with self.assertRaises(
            securesystemslib.exceptions.UnsupportedLibraryError
        ):
            securesystemslib.keys.import_ecdsakey_from_private_pem(priv)

    def test_purepy_ed25519(self):
        data = b"The quick brown fox jumps over the lazy dog"
        pub = b"\xbe\xb7\\&\x82\x06UN\x96<E\xdc\xbf<\x00A@\xd1\xcbi\xbb\xfe\x93p\xefso\x198\x8c\x9b&"
        sig = b".\xc7\xa5\xe2\x95\xfabe\xe1\x0f=\xa7\xf1\xa42\xe7t/\x04\x1f\x08\x1bO\xae\xca\xb3\xa1+\xf0\xfc\x8f6l\x91\x9c\x90\xc2g\xe9\xed\x1d\xfd\xebzuV\xb9Y\xa9m\xd0\xdc\xfe\xa1}\xa3Xb-9\xaf6\xbf\t"

        valid = securesystemslib.ed25519_keys.verify_signature(
            pub, "ed25519", sig, data
        )
        self.assertEqual(True, valid)

        bsig = b"\xd3/\x7f\x7f\xa5;6Pq\x14f]\x8b\x0e@\x8a:\xc2\xa1\xb8\xee\x11\xef\x06s\x12\xa9\x0b0\xe9@\xd5Q\xb6\xf7\xe7\xb9\xf6\xc7J\x99_L\x01\xf7\xcdi\x05\xea\xdf\x05D\x12\x1f\xeeT\xe1y\xb1\x9a\x8e\xebS\x04"
        invalid = securesystemslib.ed25519_keys.verify_signature(
            pub, "ed25519", bsig, data
        )
        self.assertEqual(False, invalid)

    def test_gpg_functions(self):
        """Public GPG functions must raise error on missing cryptography lib."""
        expected_error = securesystemslib.exceptions.UnsupportedLibraryError
        expected_error_msg = securesystemslib.gpg.functions.NO_CRYPTO_MSG

        with self.assertRaises(expected_error) as ctx:
            securesystemslib.gpg.functions.create_signature("bar")
        self.assertEqual(expected_error_msg, str(ctx.exception))

        with self.assertRaises(expected_error) as ctx:
            securesystemslib.gpg.functions.verify_signature(None, "f00", "bar")
        self.assertEqual(expected_error_msg, str(ctx.exception))

        with self.assertRaises(expected_error) as ctx:
            securesystemslib.gpg.functions.export_pubkey("f00")
        self.assertEqual(expected_error_msg, str(ctx.exception))


if __name__ == "__main__":
    unittest.main(verbosity=1, buffer=True)



#!/usr/bin/env python

"""
<Program Name>
  test_schema.py

<Author>
  Vladimir Diaz <vladimir.v.diaz@gmail.com>

<Started>
  October 2012.

<Copyright>
  See LICENSE for licensing information.

<Purpose>
  Unit test for 'schema.py'
"""

import re
import unittest

import securesystemslib.exceptions
import securesystemslib.schema as SCHEMA


class TestSchema(unittest.TestCase):  # pylint: disable=missing-class-docstring
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_Schema(self):
        # Test conditions for the instantation of classes that inherit
        # from class Schema().
        class NewSchema(SCHEMA.Schema):  # pylint: disable=abstract-method
            def __init__(self):
                pass

        new_schema = NewSchema()
        self.assertRaises(NotImplementedError, new_schema.matches, "test")

        # Define a new schema.
        class NewSchema2(SCHEMA.Schema):
            def __init__(self, string):
                self._string = string

            def check_match(self, object):  # pylint: disable=redefined-builtin
                if self._string != object:
                    message = "Expected: " + repr(self._string)
                    raise securesystemslib.exceptions.FormatError(message)

        new_schema2 = NewSchema2("test")
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            new_schema2.check_match,
            "bad",
        )
        self.assertFalse(new_schema2.matches("bad"))
        self.assertTrue(new_schema2.matches("test"))

        # Test conditions for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            new_schema2.check_match,
            True,
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            new_schema2.check_match,
            NewSchema2,
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            new_schema2.check_match,
            123,
        )

        self.assertFalse(new_schema2.matches(True))
        self.assertFalse(new_schema2.matches(NewSchema2))
        self.assertFalse(new_schema2.matches(123))

    def test_Any(self):
        # Test conditions for valid arguments.
        any_schema = SCHEMA.Any()

        self.assertTrue(any_schema.matches("test"))
        self.assertTrue(any_schema.matches(123))
        self.assertTrue(any_schema.matches(["test"]))
        self.assertTrue(any_schema.matches({"word": "definition"}))
        self.assertTrue(any_schema.matches(True))

    def test_String(self):
        # Test conditions for valid arguments.
        string_schema = SCHEMA.String("test")

        self.assertTrue(string_schema.matches("test"))

        # Test conditions for invalid arguments.
        self.assertFalse(string_schema.matches(True))
        self.assertFalse(string_schema.matches(["test"]))
        self.assertFalse(string_schema.matches(SCHEMA.Schema))

        # Test conditions for invalid arguments in a schema definition.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.String, 1
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.String, [1]
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.String, {"a": 1}
        )

    def test_AnyString(self):
        # Test conditions for valid arguments.
        anystring_schema = SCHEMA.AnyString()

        self.assertTrue(anystring_schema.matches(""))
        self.assertTrue(anystring_schema.matches("a string"))

        # Test conditions for invalid arguments.
        self.assertFalse(anystring_schema.matches(["a"]))
        self.assertFalse(anystring_schema.matches(3))
        self.assertFalse(anystring_schema.matches({"a": "string"}))

    def test_AnyNonemptyString(self):
        anynonemptystring_schema = SCHEMA.AnyNonemptyString()

        self.assertTrue(anynonemptystring_schema.matches("foo"))

        # Test conditions for invalid arguments.
        self.assertFalse(anynonemptystring_schema.matches(""))
        self.assertFalse(anynonemptystring_schema.matches(["a"]))
        self.assertFalse(anynonemptystring_schema.matches(3))
        self.assertFalse(anynonemptystring_schema.matches({"a": "string"}))

    def test_OneOf(self):
        # Test conditions for valid arguments.
        oneof_schema = SCHEMA.OneOf(
            [
                SCHEMA.ListOf(SCHEMA.Integer()),
                SCHEMA.String("Hello"),
                SCHEMA.String("bye"),
            ]
        )

        self.assertTrue(oneof_schema.matches([]))
        self.assertTrue(oneof_schema.matches("bye"))
        self.assertTrue(oneof_schema.matches([1, 2]))

        # Test conditions for invalid arguments.
        self.assertFalse(oneof_schema.matches(3))
        self.assertFalse(oneof_schema.matches(["Hi"]))

        # Test conditions for invalid arguments in a schema definition.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.OneOf, 1
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.OneOf, [1]
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.OneOf, {"a": 1}
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            SCHEMA.OneOf,
            [SCHEMA.AnyString(), 1],
        )

    def test_AllOf(self):
        # Test conditions for valid arguments.
        allof_schema = SCHEMA.AllOf(
            [SCHEMA.Any(), SCHEMA.AnyString(), SCHEMA.String("a")]
        )

        self.assertTrue(allof_schema.matches("a"))

        # Test conditions for invalid arguments.
        self.assertFalse(allof_schema.matches("b"))

        # Test conditions for invalid arguments in a schema definition.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.AllOf, 1
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.AllOf, [1]
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.AllOf, {"a": 1}
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            SCHEMA.AllOf,
            [SCHEMA.AnyString(), 1],
        )

    def test_Boolean(self):
        # Test conditions for valid arguments.
        boolean_schema = SCHEMA.Boolean()

        self.assertTrue(
            boolean_schema.matches(True) and boolean_schema.matches(False)
        )

        # Test conditions for invalid arguments.
        self.assertFalse(boolean_schema.matches(11))

    def test_ListOf(self):
        # Test conditions for valid arguments.
        listof_schema = SCHEMA.ListOf(SCHEMA.RegularExpression("(?:..)*"))
        listof2_schema = SCHEMA.ListOf(
            SCHEMA.Integer(), min_count=3, max_count=10
        )

        self.assertTrue(listof_schema.matches([]))
        self.assertTrue(
            listof_schema.matches(
                ["Hi", "this", "list", "is", "full", "of", "even", "strs"]
            )
        )

        self.assertTrue(listof2_schema.matches([3] * 3))
        self.assertTrue(listof2_schema.matches([3] * 10))

        # Test conditions for invalid arguments.
        self.assertFalse(listof_schema.matches("hi"))
        self.assertFalse(listof_schema.matches({}))
        self.assertFalse(listof_schema.matches(["This", "one", "is not"]))

        self.assertFalse(listof2_schema.matches([3] * 2))
        self.assertFalse(listof2_schema.matches(([3] * 11)))

        # Test conditions for invalid arguments in a schema definition.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.ListOf, 1
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.ListOf, [1]
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.ListOf, {"a": 1}
        )

    def test_Integer(self):
        # Test conditions for valid arguments.
        integer_schema = SCHEMA.Integer()

        self.assertTrue(integer_schema.matches(99))
        self.assertTrue(SCHEMA.Integer(lo=10, hi=30).matches(25))

        # Test conditions for invalid arguments.
        self.assertFalse(integer_schema.matches(False))
        self.assertFalse(integer_schema.matches("a string"))
        self.assertFalse(SCHEMA.Integer(lo=10, hi=30).matches(5))

    def test_DictOf(self):
        # Test conditions for valid arguments.
        dictof_schema = SCHEMA.DictOf(
            SCHEMA.RegularExpression(r"[aeiou]+"),
            SCHEMA.Struct([SCHEMA.AnyString(), SCHEMA.AnyString()]),
        )

        self.assertTrue(dictof_schema.matches({}))
        self.assertTrue(dictof_schema.matches({"a": ["x", "y"], "e": ["", ""]}))

        # Test conditions for invalid arguments.
        self.assertFalse(dictof_schema.matches(""))
        self.assertFalse(dictof_schema.matches({"a": ["x", 3], "e": ["", ""]}))
        self.assertFalse(
            dictof_schema.matches(
                {"a": ["x", "y"], "e": ["", ""], "d": ["a", "b"]}
            )
        )

        # Test conditions for invalid arguments in a schema definition.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.DictOf, 1, 1
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.DictOf, [1], [1]
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.DictOf, {"a": 1}, 1
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            SCHEMA.DictOf,
            SCHEMA.AnyString(),
            1,
        )

    def test_Optional(self):
        # Test conditions for valid arguments.
        optional_schema = SCHEMA.Object(
            k1=SCHEMA.String("X"), k2=SCHEMA.Optional(SCHEMA.String("Y"))
        )

        self.assertTrue(optional_schema.matches({"k1": "X", "k2": "Y"}))
        self.assertTrue(optional_schema.matches({"k1": "X"}))

        # Test conditions for invalid arguments.
        self.assertFalse(optional_schema.matches({"k1": "X", "k2": "Z"}))

        # Test conditions for invalid arguments in a schema definition.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.Optional, 1
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.Optional, [1]
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.Optional, {"a": 1}
        )

    def test_Object(self):
        # Test conditions for valid arguments.
        object_schema = SCHEMA.Object(
            a=SCHEMA.AnyString(),
            bc=SCHEMA.Struct([SCHEMA.Integer(), SCHEMA.Integer()]),
        )

        self.assertTrue(object_schema.matches({"a": "ZYYY", "bc": [5, 9]}))
        self.assertTrue(
            object_schema.matches({"a": "ZYYY", "bc": [5, 9], "xx": 5})
        )

        # Test conditions for invalid arguments.
        self.assertFalse(object_schema.matches({"a": "ZYYY", "bc": [5, 9, 3]}))
        self.assertFalse(object_schema.matches({"a": "ZYYY"}))

        # Test conditions for invalid arguments in a schema definition.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.Object, a="a"
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.Object, a=[1]
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            SCHEMA.Object,
            a=SCHEMA.AnyString(),
            b=1,
        )

        # Test condition for invalid non-dict arguments.
        self.assertFalse(object_schema.matches([{"a": "XYZ"}]))
        self.assertFalse(object_schema.matches(8))

    def test_Struct(self):
        # Test conditions for valid arguments.
        struct_schema = SCHEMA.Struct(
            [
                SCHEMA.ListOf(SCHEMA.AnyString()),
                SCHEMA.AnyString(),
                SCHEMA.String("X"),
            ]
        )
        struct2_schema = SCHEMA.Struct([SCHEMA.String("X")], allow_more=True)
        struct3_schema = SCHEMA.Struct(
            [SCHEMA.String("X"), SCHEMA.Integer()], [SCHEMA.Integer()]
        )

        self.assertTrue(struct_schema.matches([[], "Q", "X"]))

        self.assertTrue(struct2_schema.matches(["X"]))
        self.assertTrue(struct2_schema.matches(["X", "Y"]))
        self.assertTrue(struct2_schema.matches(["X", ["Y", "Z"]]))

        self.assertTrue(struct3_schema.matches(["X", 3]))
        self.assertTrue(struct3_schema.matches(["X", 3, 9]))

        # Test conditions for invalid arguments.
        self.assertFalse(struct_schema.matches(False))
        self.assertFalse(struct_schema.matches("Foo"))
        self.assertFalse(struct_schema.matches([[], "Q", "D"]))
        self.assertFalse(struct_schema.matches([[3], "Q", "X"]))
        self.assertFalse(struct_schema.matches([[], "Q", "X", "Y"]))

        self.assertFalse(struct2_schema.matches([]))
        self.assertFalse(struct2_schema.matches([["X"]]))

        self.assertFalse(struct3_schema.matches([]))
        self.assertFalse(struct3_schema.matches({}))
        self.assertFalse(struct3_schema.matches(["X"]))
        self.assertFalse(struct3_schema.matches(["X", 3, 9, 11]))
        self.assertFalse(struct3_schema.matches(["X", 3, "A"]))

        # Test conditions for invalid arguments in a schema definition.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.Struct, 1
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.Struct, [1]
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.Struct, {"a": 1}
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            SCHEMA.Struct,
            [SCHEMA.AnyString(), 1],
        )

    def test_RegularExpression(self):
        # Test conditions for valid arguments.
        # RegularExpression(pattern, modifiers, re_object, re_name).
        re_schema = SCHEMA.RegularExpression("h.*d")

        self.assertTrue(re_schema.matches("hello world"))

        # Provide a pattern that contains the trailing '$'
        re_schema_2 = SCHEMA.RegularExpression(
            pattern="abc$", modifiers=0, re_object=None, re_name="my_re"
        )

        self.assertTrue(re_schema_2.matches("abc"))

        # Test for valid optional arguments.
        compiled_re = re.compile("^[a-z].*")
        re_schema_optional = SCHEMA.RegularExpression(
            pattern="abc", modifiers=0, re_object=compiled_re, re_name="my_re"
        )
        self.assertTrue(re_schema_optional.matches("abc"))

        # Valid arguments, but the 'pattern' argument is unset (required if the
        # 're_object' is 'None'.)
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            SCHEMA.RegularExpression,
            None,
            0,
            None,
            None,
        )

        # Valid arguments, 're_name' is unset, and 'pattern' is None.  An exception
        # is not raised, but 're_name' is set to 'pattern'.
        re_schema_optional = SCHEMA.RegularExpression(
            pattern=None, modifiers=0, re_object=compiled_re, re_name=None
        )

        self.assertTrue(re_schema_optional.matches("abc"))
        self.assertTrue(
            re_schema_optional._re_name  # pylint: disable=protected-access
            == "pattern"
        )

        # Test conditions for invalid arguments.
        self.assertFalse(re_schema.matches("Hello World"))
        self.assertFalse(re_schema.matches("hello world!"))
        self.assertFalse(re_schema.matches([33, "Hello"]))

        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.RegularExpression, 8
        )

    def test_LengthString(self):
        # Test conditions for valid arguments.
        length_string = SCHEMA.LengthString(11)

        self.assertTrue(length_string.matches("Hello World"))
        self.assertTrue(length_string.matches("Hello Marty"))

        # Test conditions for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError,
            SCHEMA.LengthString,
            "hello",
        )

        self.assertFalse(length_string.matches("hello"))
        self.assertFalse(length_string.matches(8))

    def test_LengthBytes(self):
        # Test conditions for valid arguments.
        length_bytes = SCHEMA.LengthBytes(11)

        self.assertTrue(length_bytes.matches(b"Hello World"))
        self.assertTrue(length_bytes.matches(b"Hello Marty"))

        # Test conditions for invalid arguments.
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.LengthBytes, "hello"
        )
        self.assertRaises(
            securesystemslib.exceptions.FormatError, SCHEMA.LengthBytes, True
        )

        self.assertFalse(length_bytes.matches(b"hello"))
        self.assertFalse(length_bytes.matches(8))

    def test_AnyBytes(self):
        # Test conditions for valid arguments.
        anybytes_schema = SCHEMA.AnyBytes()

        self.assertTrue(anybytes_schema.matches(b""))
        self.assertTrue(anybytes_schema.matches(b"a string"))

        # Test conditions for invalid arguments.
        self.assertFalse(anybytes_schema.matches("a string"))
        self.assertFalse(anybytes_schema.matches(["a"]))
        self.assertFalse(anybytes_schema.matches(3))
        self.assertFalse(anybytes_schema.matches({"a": "string"}))


# Run the unit tests.
if __name__ == "__main__":
    unittest.main()
